////////////////////////////////////////////////////////////////////////////////
///                            MESH_Drawer.h                                 ///
////////////////////////////////////////////////////////////////////////////////

#ifndef MESH_DRAWER_H
#define MESH_DRAWER_H

#include "MESH_DrawerStateVariables.h"

#include <X3DTK/kernel.h>
#include <X3DTK/mesh_scenegraph.h>

namespace X3DTK {

enum SelectedType {NOTHING, VERTICES, EDGES, FACES};

namespace MESH {

// Processor drawing the mesh from the Mesh scene graph.

class Drawer : public X3DOnePassProcessor
{
public:
  Drawer();
  virtual ~Drawer();
  
  //  Setting parameters.
  void changeDrawPoints();
  void changeDrawEdges();
  void changeDrawFaces(); 
  inline void changeOneColorPerMesh() {stateVariables->changeOneColorPerMesh();}; 
  void setSelectedType(SelectedType selectedType);
  void setSelectedMesh(int selectedMesh);
  void setSelectedId(int selectedId);
  
  // Getting parameters.
  inline SelectedType getSelectedType() const {return selectedType;};
  inline int getSelectedMesh() const {return selectedMesh;};
  inline int getSelectedId() const {return selectedId;};
  
  // Draw methods.
  void drawMesh(X3DNode *N, bool selection = false);
  void drawSelected();
 
protected:
  DrawerStateVariables* stateVariables;
  
  bool drawPoints, drawEdges, drawFaces;
  SelectedType selectedType;
  int selectedMesh;
  int selectedId;

  // Draw methods.
  void drawVertex(const Mesh::SFVertex *vertex);
  void drawEdge(Mesh::SFEdge *edge);
  void drawFace(const Mesh::SFFace *face);
};

}
}

#endif

