////////////////////////////////////////////////////////////////////////////////
///                                MESH_DrawerStateVariables.h               ///
////////////////////////////////////////////////////////////////////////////////

#ifndef MESH_DRAWER_GLOBALVARIABLES_H
#define MESH_DRAWER_GLOBALVARIABLES_H

#include <X3DTK/mesh_scenegraph.h>

#include <list>
#include <vector>

namespace X3DTK {
namespace MESH {

// Global variables for the MeshDrawer processor.

class DrawerStateVariables : public StateVariables
{
public:
  DrawerStateVariables();

  // Initializes and finishes the traversal.
  void initTraversal();
  void finishTraversal();

  // Pushes the transformation matrix.
  void pushMatrix(const SFMatrix34f &transformation);  
  
  // Pops the transformation matrix.
  void popMatrix();

  // Saves current matrix state and returns index in saved vector.
  // Call storeMesh() before calling ths function
  int getCurrentMeshId();
  // Saves mesh and associated current matrix.
  void storeMesh(Mesh* mesh);
  
  // Get i Mesh of the model.
  inline const Mesh* getMesh(int i) const {return _meshVector[i];};
  // Gets stored transformation matrix associated with a given Mesh.
  inline SFMatrix34f getMatrix(int i) const {return _matrixVector[i];};

  // Whether or not primitive IDs are pushed.
  inline void setSelectionMode(bool selectionMode) {_selectionMode = selectionMode;};
  inline bool getSelectionMode() const {return _selectionMode;};
    
  // Which mesh primitive type is rendered
  void setPrimitiveType(int primitiveType) {_primitiveType = primitiveType;};
  inline int getPrimitiveType() const {return _primitiveType;};
  
  // Whether each mesh is displayed with a specific color
  void changeOneColorPerMesh() {_oneColorPerMesh = !_oneColorPerMesh;};
  inline bool getOneColorPerMesh() const {return _oneColorPerMesh;};
  
private: 
  std::list<SFMatrix34f> _matrixStack;
  std::vector<SFMatrix34f> _matrixVector;
  std::vector<Mesh *> _meshVector;
  bool _selectionMode;
  bool _oneColorPerMesh;
  int _primitiveType;
};

}
}

#endif
