/// This file is part of the X3DToolKit library
/// Copyright (C) 2002-2003  Yannick Le Goc (legoc@imag.fr)

/// http://www-artis.imag.fr/Members/Yannick.Legoc/X3D/

/// This library is free software; you can redistribute it and/or
/// modify it under the terms of the GNU Lesser General Public
/// License as published by the Free Software Foundation; either
/// version 2.1 of the License, or (at your option) any later version.

/// This library is distributed in the hope that it will be useful,
/// but WITHOUT ANY WARRANTY; without even the implied warranty of
/// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
/// Lesser General Public License for more details.

/// You should have received a copy of the GNU Lesser General Public
/// License along with this library; if not, write to the Free Software
/// Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
////////////////////////////////////////////////////////////////////////////////
//                            MESH_DisassemblerStateVariables.h               //
////////////////////////////////////////////////////////////////////////////////

#ifndef MESHDISASSEMBLERSTATEVARIABLES_H
#define MESHDISASSEMBLERSTATEVARIABLES_H

#include "MESH_SceneGraphTypes.h"
#include "MESH_Mesh.h"

#include <list>

namespace X3DTK {
namespace MESH {

/// StateVariables for the Disassembler module.

class DisassemblerStateVariables : public StateVariables
{
public:
  /// Constructor.
  DisassemblerStateVariables();
  /// Destructor.
  virtual ~DisassemblerStateVariables();

  /// Initializes the traversal.
  void init();
  /// Finishes the traversal.
  void finish();
  /// Sets the value of deformation.
  void setValue(float value);

  /// Pushes the transformation matrix.
  void pushMatrix(const SFMatrix34f &transformation);  
  /// Pops the transformation matrix.
  void popMatrix();

  /// Gets the current transformation matrix.
  SFMatrix34f getMatrix() const {return _matrixStack.front();};
  /// Gets the value of deformation.
  inline float getValue() const {return _value;};

private: 
  std::list<SFMatrix34f> _matrixStack;
  float _value;
};

}
}

#endif
