/// This file is part of the X3DToolKit library
/// Copyright (C) 2002-2003  Yannick Le Goc (legoc@imag.fr)

/// http://www-artis.imag.fr/Members/Yannick.Legoc/X3D/

/// This library is free software; you can redistribute it and/or
/// modify it under the terms of the GNU Lesser General Public
/// License as published by the Free Software Foundation; either
/// version 2.1 of the License, or (at your option) any later version.

/// This library is distributed in the hope that it will be useful,
/// but WITHOUT ANY WARRANTY; without even the implied warranty of
/// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
/// Lesser General Public License for more details.

/// You should have received a copy of the GNU Lesser General Public
/// License along with this library; if not, write to the Free Software
/// Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
////////////////////////////////////////////////////////////////////////////////
//                            SimpleX3DGLScene.h                              //
////////////////////////////////////////////////////////////////////////////////

#ifndef SIMPLEX3DGLSCENE_H
#define SIMPLEX3DGLSCENE_H

#include "X3DTypes.h"
#include "BBox.h"
#include "X3D_Scene.h"

namespace X3DTK {

class X3DMemReleaser;

namespace X3D {

class Loader;
class X3DBBoxUpdater;
class X3DGLBuilder;
class X3DFileValidator;

}

namespace GL {

class Scene;
class X3DRenderer;

}

/// Class defining a facade to allow an easy load and display of an X3D scene.

class SimpleX3DGLScene
{
public:
  /// Constructor.
  SimpleX3DGLScene();
  /// Destructor.
  virtual ~SimpleX3DGLScene();
  
  /// Loads ths scene from file.
  virtual void load(const char *file, bool fileValidation = true);
  /// Draw the scene in an opengl context.
  virtual void draw();
  /// Gets the Bounding Box center.
  inline SFVec3f getBBoxMin() const {return min;};
  /// Gets the Bounding Box size.
  inline SFVec3f getBBoxMax() const {return max;};
  /// Releases the memory.
  void release();

  /// Sets the loader.
  void setLoader(X3D::Loader *loader);
  /// Sets the BBoxUpdater.
  void setBBoxUpdater(X3D::X3DBBoxUpdater *bboxUpdater);  
  /// Sets the GLCreator.
  void setGLBuilder(X3D::X3DGLBuilder *builder);  
  /// Sets the GLRenderer.
  void setGLRenderer(GL::X3DRenderer *renderer);  
  /// Sets the MemReleaser.
  void setMemReleaser(X3DMemReleaser *memReleaser);  
  /// Sets the FileValidator.
  void setFileValidator(X3D::X3DFileValidator *fileValidator);  
  
  /// Gets the loader.
  inline X3D::Loader *getLoader() const {return _loader;};
  /// Gets the BBoxUpdater.
  inline X3D::X3DBBoxUpdater *getBBoxUpdater() const {return _bboxupdater;};  
  /// Gets the GLCreator.
  inline X3D::X3DGLBuilder *getGLBuilder() const {return _builder;};  
  /// Gets the GLRenderer.
  inline GL::X3DRenderer *getGLRenderer() const {return _renderer;};  
  /// Gets the MemReleaser.
  inline X3DMemReleaser *getMemReleaser() const {return _releaser;};  

protected:
  X3D::Scene *scene;
  GL::Scene *glscene;
  
  SFVec3f min;
  SFVec3f max;
  
  void loadFile(const char *file, bool fileValidation);
  void computeBBox();
  void buildGLScene();

private:
  X3D::Loader *_loader;
  X3D::X3DBBoxUpdater *_bboxupdater;
  X3D::X3DGLBuilder *_builder;
  GL::X3DRenderer *_renderer;
  X3DMemReleaser *_releaser;
};

}

#endif
