/// This file is part of the X3DToolKit library
/// Copyright (C) 2002-2003  Yannick Le Goc (legoc@imag.fr)

/// http://www-artis.imag.fr/Members/Yannick.Legoc/X3D/

/// This library is free software; you can redistribute it and/or
/// modify it under the terms of the GNU Lesser General Public
/// License as published by the Free Software Foundation; either
/// version 2.1 of the License, or (at your option) any later version.

/// This library is distributed in the hope that it will be useful,
/// but WITHOUT ANY WARRANTY; without even the implied warranty of
/// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
/// Lesser General Public License for more details.

/// You should have received a copy of the GNU Lesser General Public
/// License along with this library; if not, write to the Free Software
/// Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
////////////////////////////////////////////////////////////////////////////////
//                            X3D_TemplateMeshBuilderGeometry3DVisitor.h      //
////////////////////////////////////////////////////////////////////////////////

#ifndef TEMPLATEMESHBUILDERGEOMETRY3DVISITOR_H
#define TEMPLATEMESHBUILDERGEOMETRY3DVISITOR_H

#include "X3D_Geometry3DVisitor.h"
#include "X3D_TemplateMeshBuilderStateVariables.h"
#include "X3D_TemplateMeshBuilderTraits.h"
#include "X3D_Coordinate.h"
#include "X3D_Box.h"
#include "X3D_Cone.h"
#include "X3D_Cylinder.h"
#include "X3D_IndexedFaceSet.h"
#include "X3D_Sphere.h"
#include "MESH_TemplateMesh.h"

namespace X3DTK {
namespace X3D {

/// Visitor for the Geometry3D component of the TemplateMeshBuilder processor.

template<class MData, class VData, class EData, class FData>
class TemplateMeshBuilderGeometry3DVisitor : public Geometry3DVisitor
{
public:
  /// Constructor.
  TemplateMeshBuilderGeometry3DVisitor()
  : Geometry3DVisitor()
  {
    defineEnterFunction(&TemplateMeshBuilderGeometry3DVisitor::enterIndexedFaceSet);
    defineLeaveFunction(&TemplateMeshBuilderGeometry3DVisitor::leaveIndexedFaceSet);

    // StateVariables assignation.
    stateVariables = GraphTraversal::getInstanceOf<TemplateMeshBuilderStateVariables>();
  };
  
  /// Destructor.
  virtual ~TemplateMeshBuilderGeometry3DVisitor()
  {
  };

  /// Enters a IndexedFaceSet.
  virtual void enterIndexedFaceSet(IndexedFaceSet *I) const
  {
    //std::cout << "enterIFS" << std::endl;
    MESH::TemplateMesh<MData, VData, EData, FData> *TM = dynamic_cast<MESH::TemplateMesh<MData, VData, EData, FData> *>(stateVariables->getNode(I));
    if (TM == 0)
    {
      TM = new MESH::TemplateMesh<MData, VData, EData, FData>();
      TM->setX3DAppearanceNodeChild(stateVariables->getX3DAppearanceNode());
      
      stateVariables->addCoupleNode(I, TM);
    }
    /*else
      std::cout << "IFS already defined" << std::endl;*/
      
    stateVariables->pushNode(TM);
  };

  /// Enters a IndexedFaceSet.
  virtual void leaveIndexedFaceSet(IndexedFaceSet *I) const
  {
    stateVariables->popNode();
    
    // future optim? reserve the vertices and edges.
  
    typedef MESH::SFTemplateVertex<VData, EData, FData> SFVertex;
    typedef MESH::MFTemplateVertex<VData, EData, FData> MFVertex;
    typedef MESH::SFTemplateEdge<EData, FData, VData> SFEdge;
    typedef MESH::MFTemplateEdge<EData, FData, VData> MFEdge;
    typedef MESH::SFTemplateFace<FData, VData, EData> SFFace;
    typedef MESH::MFTemplateFace<FData, VData, EData> MFFace;
           
    MESH::TemplateMesh<MData, VData, EData, FData> *TM = dynamic_cast<MESH::TemplateMesh<MData, VData, EData, FData> *>(stateVariables->getNode(I));
    
    // filling the mesh.
    TemplateMeshBuilderGeometryTraits<MData, VData, EData, FData, IndexedFaceSet>::init(TM, I);
    
    SFBool ccw = I->getCcw();
    
    //iterating the faces
    const MFInt32 &coordIndex = I->getCoordIndex();
    MFInt32::const_iterator itCoordIndex = coordIndex.begin();
    
    while (itCoordIndex != coordIndex.end())
    {
      MFInt32 faceCoordIndex;
      while (*itCoordIndex != -1)
      {
        faceCoordIndex.push_back(*itCoordIndex);
        ++itCoordIndex;
      }
      SFFace *F = TM->createFace(faceCoordIndex, ccw);
      
      // processing the new face.
      TemplateMeshBuilderGeometryTraits<MData, VData, EData, FData, IndexedFaceSet>::processNewFace(TM, I, F);

      ++itCoordIndex;
    }
   
    // ending the algorithm.
    TemplateMeshBuilderGeometryTraits<MData, VData, EData, FData, IndexedFaceSet>::finish(TM, I);
  };
  
protected:
  TemplateMeshBuilderStateVariables *stateVariables;
};

}
}

#endif
