/// This file is part of the X3DToolKit library
/// Copyright (C) 2002-2003  Yannick Le Goc (legoc@imag.fr)

/// http://www-artis.imag.fr/Members/Yannick.Legoc/X3D/

/// This library is free software; you can redistribute it and/or
/// modify it under the terms of the GNU Lesser General Public
/// License as published by the Free Software Foundation; either
/// version 2.1 of the License, or (at your option) any later version.

/// This library is distributed in the hope that it will be useful,
/// but WITHOUT ANY WARRANTY; without even the implied warranty of
/// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
/// Lesser General Public License for more details.

/// You should have received a copy of the GNU Lesser General Public
/// License along with this library; if not, write to the Free Software
/// Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
////////////////////////////////////////////////////////////////////////////////
//                            X3D_TemplateMeshBuilderStateVariables.h         //
////////////////////////////////////////////////////////////////////////////////

#ifndef TEMPLATEMESHBUILDERGLOBALVARIABLES_H
#define TEMPLATEMESHBUILDERGLOBALVARIABLES_H

#include "StateVariables.h"
#include "Cloner.h"
#include "X3D_X3DNode.h"
#include "MESH_X3DNode.h"

#include <map>

namespace X3DTK {
namespace X3D {

class X3DAppearanceNode;
class Shape;

/// StateVariables for the TemplateMeshBuilder processor.

class TemplateMeshBuilderStateVariables : public StateVariables
{
public:
  /// Constructor.
  TemplateMeshBuilderStateVariables();
  /// Destructor.
  virtual ~TemplateMeshBuilderStateVariables();
  /// Initializes the traversal.
  void init();
  /// Finishes the traversal.
  virtual void finish();
  
  /// Enables the visit of children.
  void enableChildrenVisit();
  /// Disables the visit of children.
  void disableChildrenVisit();
  /// Sets the X3DAppearanceNode.
  void setX3DAppearanceNode(X3DAppearanceNode *N);
  /// Adds the couple.
  void addCoupleShape(Shape *S1, Shape *S2);
  
  /// Returns true if childrenVisit is enabled.
  inline bool getChildrenVisit() const {return _childrenVisit;};
  /// Gets the X3DAppearanceNode.
  inline X3DAppearanceNode *getX3DAppearanceNode() const {return _x3dAppearance;};
  /// Returns true if the Shape has already been met.
  bool hasShape(Shape *S) const;
  /// Gets the Shape cloned of S. Returns 0 if not cloned and is the father of an IndexedFaceSet.
  Shape *getShape(Shape *S) const;
  /// Clones the node.
  X3DNode *clone(X3DNode *N);
  
  /// Adds the couple of nodes.
  void addCoupleNode(SFNode N, MESH::SFNode MN);
  /// Pushes the node.
  void pushNode(MESH::SFNode N);
  /// Pops the node.
  void popNode();
    
  /// Gets the GLScene.
  inline MESH::SFNode getRoot() const {return _root;};
  /// Gets the top of the stack.
  inline MESH::SFNode getTop() const {return _nodeStack.front();};
  /// Gets the MESH::SFNode related with N. Returns 0 if N not found.
  MESH::SFNode getNode(SFNode N) const;
  
private:
  MESH::SFNode _root;
  MESH::MFNode _nodeStack;
  std::map<SFNode, MESH::SFNode> _nodeCoupleMap; 
  std::map<Shape *, Shape *> _shapeCoupleMap;
  
  bool _childrenVisit;
  X3DAppearanceNode *_x3dAppearance;
  Cloner *_cloner;
};

}
}

#endif
