#include "GL_DFSGroupingWalker.h"
#include "GL_StaticGroup.h"
#include "GL_X3DGroupingNode.h"
#include "GL_Switch.h"

#include <iostream>
using namespace std;

namespace X3DTK {
namespace GL {

DFSGroupingWalker::DFSGroupingWalker()
: X3DComponentWalker()
{
  defineComponentName("Grouping", "GL");

  defineWalkingFunction(&DFSGroupingWalker::walkStaticGroup);
  defineWalkingFunction(&DFSGroupingWalker::walkX3DGroupingNode);
  defineWalkingFunction(&DFSGroupingWalker::walkSwitch);
}

DFSGroupingWalker::~DFSGroupingWalker()
{
}

void DFSGroupingWalker::walkStaticGroup(StaticGroup *G) const
{
  visitor->enter(G);
  
  MFNode children = G->getChildren();
  for (MFNode::iterator it = children.begin(); it != children.end(); ++it)
    if ((*it != 0) && (visitor->walkOn(G, *it)))
      walker->walk(*it);
  
  visitor->leave(G);
}

void DFSGroupingWalker::walkX3DGroupingNode(X3DGroupingNode *N) const
{
  visitor->enter(N);
  
  MFNode children = N->getChildren();
  for (MFNode::iterator it = children.begin(); it != children.end(); ++it)
    if ((*it != 0) && (visitor->walkOn(N, *it)))
      walker->walk(*it);
  
  visitor->leave(N);
}

void DFSGroupingWalker::walkSwitch(Switch *S) const
{
  visitor->enter(S);
  
  MFNode children = S->getChildren();
  SFInt32 choice = S->getWhichChoice();
  
  SFInt32 count = 0;  
  for (MFNode::iterator it = children.begin(); it != children.end(); ++it)
  {
    if ((count == choice) && (*it != 0) && (visitor->walkOn(S, *it)))
    {
      walker->walk(*it);
      break;
    }
    ++count;
  }
  
  visitor->leave(S);
}


}
}
