/// This file is part of the X3DToolKit library
/// Copyright (C) 2002-2003  Yannick Le Goc (legoc@imag.fr)

/// http://www-artis.imag.fr/Members/Yannick.Legoc/X3D/

/// This library is free software; you can redistribute it and/or
/// modify it under the terms of the GNU Lesser General Public
/// License as published by the Free Software Foundation; either
/// version 2.1 of the License, or (at your option) any later version.

/// This library is distributed in the hope that it will be useful,
/// but WITHOUT ANY WARRANTY; without even the implied warranty of
/// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
/// Lesser General Public License for more details.

/// You should have received a copy of the GNU Lesser General Public
/// License along with this library; if not, write to the Free Software
/// Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
////////////////////////////////////////////////////////////////////////////////
//                            GL_Transform.h                                  //
////////////////////////////////////////////////////////////////////////////////

#ifndef GLTRANSFORM_H
#define GLTRANSFORM_H

#include "GL_X3DGroupingNode.h"

#include <vector>

namespace X3DTK {
namespace GL {

/// Class providing a implementation of the X3DGLNode corresponding to the X3DNode 
/// Transform.

class Transform : public X3DGroupingNode
{
public:  
  /// Constructor.
  Transform();
  /// Clones the node.
  virtual SFNode clone() const;
  /// Destructor.
  virtual ~Transform();

  /// Sets the transform matrix attribute.
  void setTransformMatrix(const MFFloat &transformMatrix);
  
  /// Gets the transform matrix attribute.
  inline const MFFloat &getTransformMatrix() const {return _transformMatrix;};
  
  /// Updates the attributes of the node.
  virtual void update();
  
protected:
  /// Copy constructor.	             
  Transform(const Transform &T);

private:  
  /// Transform matrix attribute.
  MFFloat _transformMatrix;
};

}
}

#endif
