/// This file is part of the X3DToolKit library
/// Copyright (C) 2002-2003  Yannick Le Goc (legoc@imag.fr)

/// http://www-artis.imag.fr/Members/Yannick.Legoc/X3D/

/// This library is free software; you can redistribute it and/or
/// modify it under the terms of the GNU Lesser General Public
/// License as published by the Free Software Foundation; either
/// version 2.1 of the License, or (at your option) any later version.

/// This library is distributed in the hope that it will be useful,
/// but WITHOUT ANY WARRANTY; without even the implied warranty of
/// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
/// Lesser General Public License for more details.

/// You should have received a copy of the GNU Lesser General Public
/// License along with this library; if not, write to the Free Software
/// Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
////////////////////////////////////////////////////////////////////////////////
//                            MESH_SFTemplateVertex.h                         //
////////////////////////////////////////////////////////////////////////////////

#ifndef TEMPLATEVERTEX_H
#define TEMPLATEVERTEX_H

#include "MESH_SceneGraphTypes.h"

#include <vector>

namespace X3DTK {
namespace MESH {

template<class A, class B, class C> class SFTemplateEdge;
template<class A, class B, class C> class SFTemplateFace;

/// Template class providing a Vertex class for the Mesh structure. This class can only
/// been constructed by a TemplateMesh.

template<class VData, class EData, class FData>
class SFTemplateVertex
{
  template<class A, class B, class C> friend class TemplateVertexSet;
  template<class A, class B, class C, class D> friend class TemplateMesh;
public:
  /// Gets the index.
  inline unsigned int getIndex() const {return _index;};
  /// Gets the rounding edges.
  inline const MFTemplateEdge<EData, FData, VData> &getEdges() const {return _edges;};
  /// Gets the rounding faces.
  inline const MFTemplateFace<FData, VData, EData> &getFaces() const {return _faces;};
  /// Gets the data.
  inline VData &data() {return _data;};
  /// Gets the data.
  inline const VData &data() const {return _data;};
  
private:
  const unsigned int _index;
  MFTemplateEdge<EData, FData, VData> _edges;
  MFTemplateFace<FData, VData, EData> _faces;
  VData _data;

  /// Constructor.
  SFTemplateVertex(unsigned int i)
  : _index(i) {};
  
  /// Destructor.
  ~SFTemplateVertex()
  {};
  
  /// Adds an edge.
  void addEdge(SFTemplateEdge<EData, FData, VData> *E) {_edges.push_back(E);};
  /// Adds a face.
  void addFace(SFTemplateFace<FData, VData, EData> *F) {_faces.push_back(F);};
};

}
}

#endif
