#include "MESH_X3DGroupingNode.h"
#include "X3D_X3DNode.h"

#include <iostream>
#include <algorithm>

using namespace std;

namespace X3DTK {
namespace MESH {

X3DGroupingNode::X3DGroupingNode()
: X3DNode()
{
  defineTypeName("X3DGroupingNode");
}

X3DGroupingNode::X3DGroupingNode(const X3DGroupingNode &N)
: X3DNode(N)
{
  MFAbstractNode childrenList = N.getChildrenList();
  for (MFAbstractNode::const_iterator it = childrenList.begin(); it != childrenList.end(); ++it)
    setChild(*it);
}

X3DGroupingNode::~X3DGroupingNode()
{
  for (MFNode::iterator it = _children.begin(); it != _children.end(); ++it)
    removeParentFromChild(this, *it);
}

void X3DGroupingNode::addX3DNodeChild(X3D::X3DNode *N)
{
  _x3dChildren.push_back(N);
}

MFAbstractNode X3DGroupingNode::getChildrenList() const
{
  MFAbstractNode childrenList;
  for (MFNode::const_iterator it = _children.begin(); it != _children.end(); ++it)
    if (*it != 0)
      childrenList.push_back(*it);
    
  return childrenList; 
}

bool X3DGroupingNode::addChild(const SFAbstractNode &N)
{
  if (dynamic_cast<X3DNode *>(N) != 0)
  {
    MFNode::iterator res = find(_children.begin(), _children.end(), static_cast<SFNode>(N));
    if (res == _children.end())
    {
      _children.push_back(static_cast<SFNode>(N));
      addParentToChild(this, N);
      return true;
    }
    return false;    
  }
  if (N != 0)
    cx3d << "MESH::X3DGroupingNode::addChild : a node of type " << N->getTypeName() << " cannot be a child!" << endl;
  else
    cx3d << "MESH::X3DGroupingNode::addChild : cannot add a NULL node!" << endl;
  
  return false;    
}

bool X3DGroupingNode::setChild(const SFAbstractNode &N)
{
  if (dynamic_cast<X3DNode *>(N) != 0)
  {
    MFNode::iterator res = find(_children.begin(), _children.end(), static_cast<SFNode>(N));
    if (res == _children.end())
    {
      _children.push_back(static_cast<SFNode>(N));
      addParentToChild(this, N);    
      return true;
    }
    return false;   
  }
  if (N != 0)
    cx3d << "MESH::X3DGroupingNode::setChild : a node of type " << N->getTypeName() << " cannot be a child!" << endl;
  else
    cx3d << "MESH::X3DGroupingNode::setChild : cannot add a NULL node!" << endl;
  
  return false;    
}

bool X3DGroupingNode::removeChild(const SFAbstractNode &N)
{
  MFNode::iterator res = find(_children.begin(), _children.end(), N);
  if (res != _children.end())
  {
    removeParentFromChild(this, N);
    _children.erase(res);
    return true;
  }  
    
  return false;
}

}
}
