#include "X3D_OrientationInterpolator.h"

#include <iostream>

using namespace std;

namespace X3DTK {
namespace X3D {

OrientationInterpolator::OrientationInterpolator()
  : X3DInterpolatorNode(),
    _keyValue(MFRotation())
{
  defineTypeName("OrientationInterpolator", "Interpolation");
}

OrientationInterpolator::OrientationInterpolator(const MFFloat &key,
                                           const MFRotation &keyValue)
  : X3DInterpolatorNode(key),
    _keyValue(keyValue)
{
  defineTypeName("OrientationInterpolator", "Interpolation");
}

OrientationInterpolator::OrientationInterpolator(const OrientationInterpolator &N)
  : X3DInterpolatorNode(N),
    _keyValue(N._keyValue)
{
}

SFAbstractNode OrientationInterpolator::clone() const
{
  return new OrientationInterpolator(*this);
}

OrientationInterpolator::~OrientationInterpolator()
{
}

void OrientationInterpolator::setKeyValue(const MFRotation &keyValue)
{
  _keyValue = keyValue;
}

void OrientationInterpolator::load(const X3DFileElement *element)
{
  X3DInterpolatorNode::load(element);

  int index;
  index = element->getIndexAttribute("keyValue");
  if (index != -1)
    _keyValue = MFRotation(element->getAttribute(index));
}

SFString &OrientationInterpolator::write(SFString &output) const
{
  X3DInterpolatorNode::write(output);
  
  if (!_keyValue.empty())
    output += " keyValue=\"" + toSFString(_keyValue) + "\"";

  return output;  
}

}
}
