#include "X3D_TriangleSet.h"

#include <iostream>

using namespace std;

namespace X3DTK {
namespace X3D {

TriangleSet::TriangleSet()
: X3DComposedGeometryNode(), _convex(true), _creaseAngle(0.0f)
{
  defineTypeName("TriangleSet", "Rendering");
}

TriangleSet::TriangleSet(X3DColorNode *color, X3DCoordinateNode *coord, X3DNormalNode *normal, X3DTextureCoordinateNode *texCoord, const SFBool &ccw, const SFBool &colorPerVertex, const SFBool &convex, const SFFloat &creaseAngle, const SFBool &normalPerVertex, const SFBool &solid)
: X3DComposedGeometryNode(color, coord, normal, texCoord, ccw, colorPerVertex, normalPerVertex, solid),
_convex(convex), _creaseAngle(creaseAngle)
{
  defineTypeName("TriangleSet", "Rendering");
}

TriangleSet::TriangleSet(const TriangleSet &T)
: X3DComposedGeometryNode(T), _convex(T._convex), _creaseAngle(T._creaseAngle)
{
}

SFAbstractNode TriangleSet::clone() const
{
  return new TriangleSet(*this);
}

TriangleSet::~TriangleSet()
{
}

void TriangleSet::setConvex(const SFBool &convex)
{
  _convex = convex;
}

void TriangleSet::setCreaseAngle(const SFFloat &creaseAngle)
{
  _creaseAngle = creaseAngle;
}

void TriangleSet::load(const X3DFileElement *element)
{
  X3DComposedGeometryNode::load(element);

  int index;  
  index = element->getIndexAttribute("convex");  
  if (index != -1)
    _convex = (element->getAttribute(index).upper() == "TRUE");

  index = element->getIndexAttribute("creaseAngle");
  if (index != -1)
    _creaseAngle = element->getAttribute(index).toFloat(); 
}    
    
SFString &TriangleSet::write(SFString &output) const
{
  X3DComposedGeometryNode::write(output);
  
  if (!_convex)  
    output += " convex=\"FALSE\"";
  if (_creaseAngle != 0.0f)
    output += " creaseAngle=\"" + toSFString(_creaseAngle) + "\"";
    
  return output;
}

}
}
