#include "MESH_MeshNormalUpdaterFacesCoreVisitor.h"

using namespace std;

namespace X3DTK {
namespace MESH {

MeshNormalUpdaterFacesCoreVisitor::MeshNormalUpdaterFacesCoreVisitor()
{
  defineEnterFunction(&MeshNormalUpdaterFacesCoreVisitor::enterMesh);
}

void MeshNormalUpdaterFacesCoreVisitor::enterMesh(Mesh *M) const
{
  // Compute all faces normals 
  const Mesh::MFFace &faces = M->getFaces();
  for (Mesh::MFFace::const_iterator f = faces.begin(), end = faces.end() ; f != end; ++f)
  {
    SFVec3f normal;
    const Mesh::MFEdge& edges = (*f)->getEdges();
    Mesh::MFEdge::const_iterator e = edges.begin();
    // Choose a pivot point
    const SFPoint3f &firstPoint = (*e)->getFromVertex()->data().getPoint();

    // Initialize secondPoint from second edge
    ++e;
    SFPoint3f secondPoint = (*e)->getFromVertex()->data().getPoint();

    // Start loop at third edge
    ++e;
    for (Mesh::MFEdge::const_iterator end = edges.end() ; e != end; ++e)
    {
      SFPoint3f thirdPoint = (*e)->getFromVertex()->data().getPoint();
      normal = normal + crossprod(secondPoint - firstPoint, thirdPoint - firstPoint);
      secondPoint = thirdPoint;
    }

    // Update face normal (if face does not already has a normal)
    if ((*f)->data().getNormal().norm() < 1E-4)
      (*f)->data().setNormal(normal.normalize());
  }
}

} 
} 
