/// This file is part of the X3DToolKit library
/// Copyright (C) 2002-2003  Yannick Le Goc (legoc@imag.fr)

/// http://www-artis.imag.fr/Members/Yannick.Legoc/X3D/

/// This library is free software; you can redistribute it and/or
/// modify it under the terms of the GNU Lesser General Public
/// License as published by the Free Software Foundation; either
/// version 2.1 of the License, or (at your option) any later version.

/// This library is distributed in the hope that it will be useful,
/// but WITHOUT ANY WARRANTY; without even the implied warranty of
/// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
/// Lesser General Public License for more details.

/// You should have received a copy of the GNU Lesser General Public
/// License along with this library; if not, write to the Free Software
/// Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
////////////////////////////////////////////////////////////////////////////////
//                            X3D_GLBuilderStateVariables.h                   //
////////////////////////////////////////////////////////////////////////////////

#ifndef GLBUILDERGLOBALVARIABLES_H
#define GLBUILDERGLOBALVARIABLES_H

#include "StateVariables.h"
#include "GL_X3DNode.h"

#include <map>

namespace X3DTK {
namespace X3D {

/// StateVariables for the GLBuilder module.

class GLBuilderStateVariables : public StateVariables
{
public:
  /// Constructor.
  GLBuilderStateVariables();
  /// Destructor.
  virtual ~GLBuilderStateVariables();
  /// Initializes the traversal.
  virtual void init();
  /// Finishes the traversal.
  virtual void finish();
  
  /// Adds the couple of nodes.
  void addCoupleNode(SFNode N, GL::SFNode GN);
  /// Pushes the node.
  void pushNode(GL::SFNode N);
  /// Pops the node.
  void popNode();
    
  /// Gets the GLScene.
  inline GL::SFNode getRoot() const {return _root;};
  /// Gets the top of the stack.
  inline GL::SFNode getTop() const {return (!_nodeStack.empty() ? _nodeStack.front() : 0);};
  /// Gets the GL::SFNode related with N. Returns 0 if N not found.
  GL::SFNode getNode(SFNode N) const;
  
private:
  GL::SFNode _root;
  GL::MFNode _nodeStack;
  std::map<SFNode, GL::SFNode> _nodeCoupleMap;
  char *_cwd;
};

}
}

#endif
