#include "X3D_TemplateMeshBuilderGroupingVisitor.h"
#include "X3D_X3DGroupingNode.h"
#include "X3D_StaticGroup.h"
#include "X3D_Transform.h"
#include "X3D_Switch.h"
#include "MESH_Transform.h"
#include "MESH_Switch.h"

#include <iostream>

using namespace std;

namespace X3DTK {
namespace X3D {

TemplateMeshBuilderGroupingVisitor::TemplateMeshBuilderGroupingVisitor()
: GroupingVisitor()
{
  // Enter functions.
  defineEnterFunction(&TemplateMeshBuilderGroupingVisitor::enterX3DChildNode);
  defineEnterFunction(&TemplateMeshBuilderGroupingVisitor::enterX3DGroupingNode);
  defineEnterFunction(&TemplateMeshBuilderGroupingVisitor::enterStaticGroup);
  defineEnterFunction(&TemplateMeshBuilderGroupingVisitor::enterTransform);
  defineEnterFunction(&TemplateMeshBuilderGroupingVisitor::enterSwitch);

  defineLeaveFunction(&TemplateMeshBuilderGroupingVisitor::leaveX3DChildNode);
  defineLeaveFunction(&TemplateMeshBuilderGroupingVisitor::leaveX3DGroupingNode);
  
  // StateVariables assignation.
  stateVariables = GraphTraversal::getInstanceOf<TemplateMeshBuilderStateVariables>();
}

TemplateMeshBuilderGroupingVisitor::~TemplateMeshBuilderGroupingVisitor()
{
}

void TemplateMeshBuilderGroupingVisitor::enterX3DChildNode(X3DChildNode *N) const
{
  stateVariables->disableChildrenVisit();
  X3DNode *NN = static_cast<X3DNode *>(stateVariables->clone(N));
  
  MESH::X3DGroupingNode *G = dynamic_cast<MESH::X3DGroupingNode *>(stateVariables->getTop());
  if (G != 0)
    G->addX3DNodeChild(NN);
}

void TemplateMeshBuilderGroupingVisitor::enterX3DGroupingNode(X3DGroupingNode *N) const
{
  MESH::SFNode MN = stateVariables->getNode(N);
  if (MN == 0)
  {
    MN = new MESH::Transform();  
    stateVariables->addCoupleNode(N, MN);
  }
  stateVariables->pushNode(MN);
}

void TemplateMeshBuilderGroupingVisitor::enterStaticGroup(StaticGroup *S) const
{
  MESH::SFNode MS = stateVariables->getNode(S);
  if (MS == 0)
  {
    MS = new MESH::Transform();  
    stateVariables->addCoupleNode(S, MS);
  }
  stateVariables->pushNode(MS);
}

void TemplateMeshBuilderGroupingVisitor::enterTransform(Transform *T) const
{
  MESH::Transform *MT = dynamic_cast<MESH::Transform *>(stateVariables->getNode(T));
  if (MT == 0)
  {
    MT = new MESH::Transform();
    MT->setCenter(T->getCenter());
    MT->setRotation(T->getRotation());
    MT->setScale(T->getScale());
    MT->setScaleOrientation(T->getScaleOrientation());
    MT->setTranslation(T->getTranslation());
      
    stateVariables->addCoupleNode(T, MT);
  }
  stateVariables->pushNode(MT);
}

void TemplateMeshBuilderGroupingVisitor::enterSwitch(Switch *S) const
{
  MESH::SFNode MS = stateVariables->getNode(S);
  if (MS == 0)
  {
    MS = new MESH::Switch();  
    stateVariables->addCoupleNode(S, MS);
  }
  stateVariables->pushNode(MS);
}

void TemplateMeshBuilderGroupingVisitor::leaveX3DChildNode(X3DChildNode *N) const
{
  stateVariables->enableChildrenVisit();
}

void TemplateMeshBuilderGroupingVisitor::leaveX3DGroupingNode(X3DGroupingNode *N) const
{
  stateVariables->popNode();
}


}
}
