#include "X3D_TemplateMeshBuilderStateVariables.h"
#include "X3D_X3DAppearanceNode.h"
#include "X3D_Shape.h"

#include <iostream>

using namespace std;

namespace X3DTK {
namespace X3D {

TemplateMeshBuilderStateVariables::TemplateMeshBuilderStateVariables()
: StateVariables(), _root(0), _childrenVisit(true), _x3dAppearance(0), _cloner(new Cloner())
{
}

TemplateMeshBuilderStateVariables::~TemplateMeshBuilderStateVariables()
{
  delete _cloner;
}

void TemplateMeshBuilderStateVariables::init()
{
  _root = 0;
  _nodeStack.push_front(0); 
  _childrenVisit = true;
  _x3dAppearance = 0;
}

void TemplateMeshBuilderStateVariables::finish()
{
  _nodeStack.clear();
  _nodeCoupleMap.clear();
}

void TemplateMeshBuilderStateVariables::enableChildrenVisit()
{
  _childrenVisit = true;
}

void TemplateMeshBuilderStateVariables::disableChildrenVisit()
{
  _childrenVisit = false;
}

void TemplateMeshBuilderStateVariables::setX3DAppearanceNode(X3DAppearanceNode *N)
{
  _x3dAppearance = N;
}

void TemplateMeshBuilderStateVariables::addCoupleShape(Shape *S1, Shape *S2)
{
  _shapeCoupleMap[S1] = S2;
}

bool TemplateMeshBuilderStateVariables::hasShape(Shape *S) const
{
  map<Shape *, Shape *>::const_iterator it = _shapeCoupleMap.find(S);
  if (it != _shapeCoupleMap.end())
    return true;
    
  return false;  
}

Shape *TemplateMeshBuilderStateVariables::getShape(Shape *S) const
{
  map<Shape *, Shape *>::const_iterator it = _shapeCoupleMap.find(S);
  if (it != _shapeCoupleMap.end())
    return (*it).second;
    
  return 0;  
}

X3DNode *TemplateMeshBuilderStateVariables::clone(X3DNode *N)
{
  return static_cast<X3DNode *>(_cloner->clone(N));
}
  
void TemplateMeshBuilderStateVariables::addCoupleNode(SFNode N, MESH::SFNode MN)
{
  _nodeCoupleMap[N] = MN;  
  
  SFString name = X3DAbstractNode::DEFDict.getNameOfNode(N);
  if (name != "")
    X3DAbstractNode::DEFDict.insert(name, MN);
  
  // init root node.
  if (_root == 0)
    _root = MN;
}

void TemplateMeshBuilderStateVariables::pushNode(MESH::SFNode N)
{
  MESH::SFNode T = getTop();
  if (T != 0)
  {
    T->addChild(N);
  }
  _nodeStack.push_front(N);
}

void TemplateMeshBuilderStateVariables::popNode()
{
  _nodeStack.pop_front();
}

MESH::SFNode TemplateMeshBuilderStateVariables::getNode(SFNode N) const
{
  map<SFNode, MESH::SFNode>::const_iterator it = _nodeCoupleMap.find(N);
  if (it != _nodeCoupleMap.end())
    return (*it).second;
    
  return 0;  
}

}
}
