/// This file is part of the X3DToolKit library
/// Copyright (C) 2002-2003  Yannick Le Goc (legoc@imag.fr)

/// http://www-artis.imag.fr/Members/Yannick.Legoc/X3D/

/// This library is free software; you can redistribute it and/or
/// modify it under the terms of the GNU Lesser General Public
/// License as published by the Free Software Foundation; either
/// version 2.1 of the License, or (at your option) any later version.

/// This library is distributed in the hope that it will be useful,
/// but WITHOUT ANY WARRANTY; without even the implied warranty of
/// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
/// Lesser General Public License for more details.

/// You should have received a copy of the GNU Lesser General Public
/// License along with this library; if not, write to the Free Software
/// Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
////////////////////////////////////////////////////////////////////////////////
//                            SFSceneGraph.h                                  //
////////////////////////////////////////////////////////////////////////////////

#ifndef SFSCENEGRAPH_H
#define SFSCENEGRAPH_H

#include "X3DTypes.h"

namespace X3DTK {

/// Class representing the concept of SFSceneGraph. 

class SFSceneGraph
{
  friend class SFComponent;
  
public:
  /// Gets the SFSceneGraph of name name.
  static SFSceneGraph *getSceneGraph(const SFString &name);
  
  /// Returns the name of the SFSceneGraph.
  inline SFString getName() const {return _name;};
  /// Gets the list of the SFTypes.
  inline MFComponent getComponentList() const {return _componentList;};

private:  
  SFSceneGraph(const SFString &name);
  ~SFSceneGraph();
  
  /// Adds one SFType.
  void addComponent(SFComponent *component);
  /// Adds one reference.
  void addOneReference();
  /// Removes one reference of the SFSceneGraph.
  static void removeOneReference(SFSceneGraph *sceneGraph);
  
  SFString _name;    
  int _refCount;
  MFComponent _componentList;
  
  static MFSceneGraph _sceneGraphList;
};

}

#endif
