/// This file is part of the X3DToolKit library
/// Copyright (C) 2002-2003  Yannick Le Goc (legoc@imag.fr)

/// http://www-artis.imag.fr/Members/Yannick.Legoc/X3D/

/// This library is free software; you can redistribute it and/or
/// modify it under the terms of the GNU Lesser General Public
/// License as published by the Free Software Foundation; either
/// version 2.1 of the License, or (at your option) any later version.

/// This library is distributed in the hope that it will be useful,
/// but WITHOUT ANY WARRANTY; without even the implied warranty of
/// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
/// Lesser General Public License for more details.

/// You should have received a copy of the GNU Lesser General Public
/// License along with this library; if not, write to the Free Software
/// Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
////////////////////////////////////////////////////////////////////////////////
//                            LeaveFunction.h                                 //
////////////////////////////////////////////////////////////////////////////////

#ifndef LEAVEFUNCTION_H
#define LEAVEFUNCTION_H

#include "X3DActiveFunction.h"
#include "X3DTypes.h"

namespace X3DTK {

class X3DComponentVisitor;
/// Pointer to LeaveFunction.
typedef void (X3DComponentVisitor::* ptrToLeaveFunction)(SFAbstractNode) const;

/// Abstract class defining the concept of leave function implemented by a functor.

class LeaveFunction : public X3DActiveFunction
{
public:
  /// Constructor.
  LeaveFunction(const ptrToLeaveFunction leave, const X3DComponentVisitor *component);
  /// Destructor.
  ~LeaveFunction();
  
  /// Leave the node.
  inline void leave(SFAbstractNode N) const {(component_->*ptr2leave_)(N);};
  
private:
  const ptrToLeaveFunction ptr2leave_;
  const X3DComponentVisitor *component_;
};

}

#endif
