/// This file is part of the X3DToolKit library
/// Copyright (C) 2002-2003  Yannick Le Goc (legoc@imag.fr)

/// http://www-artis.imag.fr/Members/Yannick.Legoc/X3D/

/// This library is free software; you can redistribute it and/or
/// modify it under the terms of the GNU Lesser General Public
/// License as published by the Free Software Foundation; either
/// version 2.1 of the License, or (at your option) any later version.

/// This library is distributed in the hope that it will be useful,
/// but WITHOUT ANY WARRANTY; without even the implied warranty of
/// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
/// Lesser General Public License for more details.

/// You should have received a copy of the GNU Lesser General Public
/// License along with this library; if not, write to the Free Software
/// Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
////////////////////////////////////////////////////////////////////////////////
//                            X3DProcessor.h                                  //
////////////////////////////////////////////////////////////////////////////////

#ifndef X3DPROCESSOR_H
#define X3DPROCESSOR_H

#include "GraphTraversal.h"

#include <list>
#include <typeinfo>

#include <iostream>

namespace X3DTK {

/// Abstract class defining a concept of processor. The class is also a X3DProcessor factory.

class X3DProcessor
{
public:
  /// Constructor.
  X3DProcessor();
  /// Destructor.
  virtual ~X3DProcessor() = 0;

  /// Gets the instance of X3DProcessor.
  template <class P>
  static P *getInstanceOf()
  {
    for (std::list<X3DProcessor *>::iterator it = _X3DProcessorList.begin(); it != _X3DProcessorList.end(); ++it)
    {
      if (typeid(*(*it)) == typeid(P))
        return static_cast<P *>(*it);       
    }
    P *p = new P();
    _X3DProcessorList.push_back(p);
    return p;
  };

  /// Removes the instance of X3DProcessor.
  template <class P>
  static void removeInstanceOf()
  {
    for (std::list<X3DProcessor *>::iterator it = _X3DProcessorList.begin(); it != _X3DProcessorList.end(); ++it)
    {
      if (typeid(*(*it)) == typeid(P))
      {
        delete *it;
        _X3DProcessorList.erase(it);
        break;
      }  
    }
  };
  
  /// Removes all the instances of X3DProcessor.
  static void removeAllInstances();
  
private:
  static std::list<X3DProcessor *> _X3DProcessorList;
};

}

#endif
