#include "X3DTypesToSFString.h"

using namespace X3DTK;

SFString X3DTK::toSFString(const SFString &DEF, const SFString &USE)
{
  SFString res("");
  if (DEF != SFString(""))
    res = res + SFString(" DEF=\"") + DEF + SFString("\"");
  if (USE != SFString(""))
    res = res + SFString(" USE=\"") + USE + SFString("\""); 
  return res;   
}

SFString X3DTK::toSFString(const MFString &ms)
{
  SFString res("");
  if (!ms.empty())
  {
    for (unsigned int i = 0; i < ms.size() - 1; ++i)
      res = res + "\"" + ms[i] + SFString("\", ");
  
    res = res + "\"" + ms.back() + "\"";        
  }  
  return res;
}

SFString X3DTK::toSFString(const SFBool b)
{
  if (b)
    return "TRUE";
  return "FALSE";  
}

SFString X3DTK::toSFString(const SFInt32 i)
{
  return SFString::number(i);
}

SFString X3DTK::toSFString(const MFInt32 &mi)
{
  SFString res("");
  if (!mi.empty())
  {
    for (unsigned int i = 0; i < mi.size() - 1; ++i)
      res = res + SFString::number(mi[i]) + SFString(" ");
  
    res = res + SFString::number(mi.back());        
  }  
  return res;
}

SFString X3DTK::toSFString(const SFFloat f)
{
  return SFString::number(f);
}

SFString X3DTK::toSFString(const MFFloat &mf)
{
  SFString res("");
  if (!mf.empty())
  {
    for (unsigned int i = 0; i < mf.size() - 1; ++i)
      res = res + SFString::number(mf[i]) + SFString(" ");
  
    res = res + SFString::number(mf.back());        
  }  
  return res;
}

SFString X3DTK::toSFString(const SFDouble d)
{
  return SFString::number(d);
} 

SFString X3DTK::toSFString(const MFDouble &md)
{
  SFString res("");
  if (!md.empty())
  {
    for (unsigned int i = 0; i < md.size() - 1; ++i)
      res = res + SFString::number(md[i]) + SFString(" ");
  
    res = res + SFString::number(md.back());        
  }  
  return res;
}

SFString X3DTK::toSFString(const SFColorRGBA c)
{
  return SFString::number(c.r) + SFString(" ") 
       + SFString::number(c.g) + SFString(" ")
       + SFString::number(c.b) + SFString(" ")
       + SFString::number(c.a);
}

SFString X3DTK::toSFString(const MFColorRGBA &mc)
{
  SFString res("");
  if (!mc.empty())
  {
    for (unsigned int i = 0; i < mc.size() - 1; ++i)
      res = res + toSFString(mc[i]) + SFString(", ");
    
    res = res + toSFString(mc.back());        
  }  
  return res;
}

SFString X3DTK::toSFString(const SFColor c)
{
  return SFString::number(c.r) + SFString(" ") 
       + SFString::number(c.g) + SFString(" ")
       + SFString::number(c.b);
}

SFString X3DTK::toSFString(const MFColor &mc)
{
  SFString res("");
  if (!mc.empty())
  {
    for (unsigned int i = 0; i < mc.size() - 1; ++i)
      res = res + toSFString(mc[i]) + SFString(", ");
    
    res = res + toSFString(mc.back());        
  }  
  return res;
}

SFString X3DTK::toSFString(const SFVec2f v)
{
  return SFString::number(v.x) + SFString(" ") 
       + SFString::number(v.y);
}

SFString X3DTK::toSFString(const MFVec2f &mv)
{
  SFString res("");
  if (!mv.empty())
  {
    for (unsigned int i = 0; i < mv.size() - 1; ++i)
      res = res + toSFString(mv[i]) + SFString(", ");
    
    res = res + toSFString(mv.back());        
  }  
  return res;
}

SFString X3DTK::toSFString(const SFVec3f v)
{
  return SFString::number(v.x) + SFString(" ") 
       + SFString::number(v.y) + SFString(" ")
       + SFString::number(v.z);
}

SFString X3DTK::toSFString(const MFVec3f &mv)
{
  SFString res("");
  if (!mv.empty())
  {
    for (unsigned int i = 0; i < mv.size() - 1; ++i)
      res = res + toSFString(mv[i]) + SFString(", ");
    
    res = res + toSFString(mv.back());        
  }  
  return res;
}

SFString X3DTK::toSFString(const SFRotation r)
{
  return SFString::number(r.x) + SFString(" ") 
       + SFString::number(r.y) + SFString(" ")
       + SFString::number(r.z) + SFString(" ")
       + SFString::number(r.angle);
}

SFString X3DTK::toSFString(const MFRotation &mr)
{
  SFString res("");
  if (!mr.empty())
  {
    for (unsigned int i = 0; i < mr.size() - 1; ++i)
      res = res + toSFString(mr[i]) + SFString(", ");
    
    res = res + toSFString(mr.back());        
  }  
  return res;
}
